from .collection import CollectionHandler
from .history import HistoryHandler
from .playback import PlaybackHandler
from .ratings import RatingsHandler
from .watchlist import WatchlistHandler


__all__ = [
    'CollectionHandler',
    'HistoryHandler',
    'PlaybackHandler',
    'RatingsHandler',
    'WatchlistHandler'
]
